/**
  ******************************************************************************
  * File Name          : main.c
  * Description        : Main program body
  ******************************************************************************
  *
  * COPYRIGHT(c) 2017 STMicroelectronics
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
  *   1. Redistributions of source code must retain the above copyright notice,
  *      this list of conditions and the following disclaimer.
  *   2. Redistributions in binary form must reproduce the above copyright notice,
  *      this list of conditions and the following disclaimer in the documentation
  *      and/or other materials provided with the distribution.
  *   3. Neither the name of STMicroelectronics nor the names of its contributors
  *      may be used to endorse or promote products derived from this software
  *      without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */
/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "stm32f0xx_hal.h"

/* USER CODE BEGIN Includes */

#include <string.h>
#include "lcd1803.h"

/* USER CODE END Includes */

/* Private variables ---------------------------------------------------------*/
SPI_HandleTypeDef hspi1;

/* USER CODE BEGIN PV */
/* Private variables ---------------------------------------------------------*/

/* Some useful macros */

#define PutString(disp, str)  LCD1803_Write(disp, (uint8_t *)str, strlen(str));
#define ClearLine(disp, line) LCD1803_DDRAMADDR(disp, line*0x20); \
LCD1803_Write(disp, (uint8_t *) "                    ", 20);

/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
void Error_Handler(void);
static void MX_GPIO_Init(void);
static void MX_SPI1_Init(void);

/* USER CODE BEGIN PFP */
/* Private function prototypes -----------------------------------------------*/

void Demo_Welcome(LCD1803_t *disp);
void Demo_Bidirect(LCD1803_t *disp);
void Demo_CursorMode(LCD1803_t *disp);
void Demo_EntryMode(LCD1803_t *disp);
void Demo_ShiftCursor(LCD1803_t *disp);
void Demo_ShiftDisplay(LCD1803_t *disp);
void Demo_AutoShift(LCD1803_t *disp);
void Demo_SmoothScroll(LCD1803_t *disp);
void Demo_DH(LCD1803_t *disp);
void Demo_Rom(LCD1803_t *disp);
void Demo_CG(LCD1803_t *disp);
void Demo_Reverse(LCD1803_t *disp);

/* USER CODE END PFP */

/* USER CODE BEGIN 0 */

/* USER CODE END 0 */

int main(void)
{

  /* USER CODE BEGIN 1 */
    LCD1803_t disp;

  /* USER CODE END 1 */

  /* MCU Configuration----------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* Configure the system clock */
  SystemClock_Config();

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  MX_SPI1_Init();

  /* USER CODE BEGIN 2 */

    /* Perform reset by pulling PA0 to low for 20ms */

    HAL_GPIO_WritePin(GPIOA, GPIO_PIN_0, GPIO_PIN_RESET);
    HAL_Delay(20);
    HAL_GPIO_WritePin(GPIOA, GPIO_PIN_0, GPIO_PIN_SET);
    HAL_Delay(20);

    /* Initialize display */

    LCD1803_Init(&disp, &hspi1);

  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */
  while (1)
  {


  /* USER CODE END WHILE */

  /* USER CODE BEGIN 3 */


    /* Show the different functions */

    Demo_Welcome(&disp);
    Demo_Bidirect(&disp);
    Demo_CursorMode(&disp);
    Demo_EntryMode(&disp);
    Demo_ShiftCursor(&disp);
    Demo_ShiftDisplay(&disp);
    Demo_AutoShift(&disp);
    Demo_SmoothScroll(&disp);
    Demo_DH(&disp);
    Demo_Rom(&disp);
    Demo_CG(&disp);
    Demo_Reverse(&disp);

  }
  /* USER CODE END 3 */

}

/** System Clock Configuration
*/
void SystemClock_Config(void)
{

  RCC_OscInitTypeDef RCC_OscInitStruct;
  RCC_ClkInitTypeDef RCC_ClkInitStruct;

    /**Initializes the CPU, AHB and APB busses clocks
    */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSI;
  RCC_OscInitStruct.HSIState = RCC_HSI_ON;
  RCC_OscInitStruct.HSICalibrationValue = 16;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSI;
  RCC_OscInitStruct.PLL.PLLMUL = RCC_PLL_MUL12;
  RCC_OscInitStruct.PLL.PREDIV = RCC_PREDIV_DIV1;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }

    /**Initializes the CPU, AHB and APB busses clocks
    */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV1;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_1) != HAL_OK)
  {
    Error_Handler();
  }

    /**Configure the Systick interrupt time
    */
  HAL_SYSTICK_Config(HAL_RCC_GetHCLKFreq()/1000);

    /**Configure the Systick
    */
  HAL_SYSTICK_CLKSourceConfig(SYSTICK_CLKSOURCE_HCLK);

  /* SysTick_IRQn interrupt configuration */
  HAL_NVIC_SetPriority(SysTick_IRQn, 0, 0);
}

/* SPI1 init function */
static void MX_SPI1_Init(void)
{

  hspi1.Instance = SPI1;
  hspi1.Init.Mode = SPI_MODE_MASTER;
  hspi1.Init.Direction = SPI_DIRECTION_2LINES;
  hspi1.Init.DataSize = SPI_DATASIZE_8BIT;
  hspi1.Init.CLKPolarity = SPI_POLARITY_HIGH;
  hspi1.Init.CLKPhase = SPI_PHASE_2EDGE;
  hspi1.Init.NSS = SPI_NSS_SOFT;
  hspi1.Init.BaudRatePrescaler = SPI_BAUDRATEPRESCALER_64;
  hspi1.Init.FirstBit = SPI_FIRSTBIT_LSB;
  hspi1.Init.TIMode = SPI_TIMODE_DISABLE;
  hspi1.Init.CRCCalculation = SPI_CRCCALCULATION_DISABLE;
  hspi1.Init.CRCPolynomial = 7;
  hspi1.Init.CRCLength = SPI_CRC_LENGTH_DATASIZE;
  hspi1.Init.NSSPMode = SPI_NSS_PULSE_DISABLE;
  if (HAL_SPI_Init(&hspi1) != HAL_OK)
  {
    Error_Handler();
  }

}

/** Configure pins as
        * Analog
        * Input
        * Output
        * EVENT_OUT
        * EXTI
*/
static void MX_GPIO_Init(void)
{

  GPIO_InitTypeDef GPIO_InitStruct;

  /* GPIO Ports Clock Enable */
  __HAL_RCC_GPIOA_CLK_ENABLE();

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOA, GPIO_PIN_0, GPIO_PIN_SET);

  /*Configure GPIO pin : PA0 */
  GPIO_InitStruct.Pin = GPIO_PIN_0;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

}

/* USER CODE BEGIN 4 */

/** \brief Prints welcome message and turning contrast on from 0 to 40
 *
 */
void Demo_Welcome(LCD1803_t *disp) {
    LCD1803_Clear(disp);
    LCD1803_Set_Contrast(disp, 0);
    LCD1803_MoveXY(disp, 6, 0);
    PutString(disp, "Welcome!");
    for(uint8_t contrast=0; contrast<=30; contrast++) {
        LCD1803_Set_Contrast(disp, contrast);
        HAL_Delay(100);
    }

    HAL_Delay(2000);
}


/** \brief Shows all possible display orientations ("bidirectional mode")
 *
 */
void Demo_Bidirect(LCD1803_t *disp) {
    LCD1803_Clear(disp);
    PutString(disp, "<00");
    LCD1803_MoveXY(disp, 17, 0);
    PutString(disp, "19>");
    LCD1803_MoveXY(disp, 0, 3);
    PutString(disp, "<96");
    LCD1803_MoveXY(disp, 16, 3);
    PutString(disp, "115>");

    ClearLine(disp, 1);
    LCD1803_MoveXY(disp, 0, 1);
    PutString(disp, "Set_BidirectMode:");

    for(int repeat=0; repeat < 5; repeat++) {
        for(uint8_t orientation=0; orientation < 4; orientation++) {
            LCD1803_MoveXY(disp, 0, 2);
            switch(orientation) {
            case 0:
                PutString(disp, "HORIZ=0, VERT=0");
                break;

            case LCD1803_BIDIRECTMODE_HORIZ:
                PutString(disp, "HORIZ=1, VERT=0");
                break;

            case LCD1803_BIDIRECTMODE_VERT:
                PutString(disp, "HORIZ=0, VERT=1");
                break;

            case LCD1803_BIDIRECTMODE_HORIZ|LCD1803_BIDIRECTMODE_VERT:
                PutString(disp, "HORIZ=1, VERT=1");
                break;
            }
            LCD1803_Set_BidirectMode(disp, orientation);
            HAL_Delay(2000);
        }
    }

    LCD1803_Set_BidirectMode(disp, LCD1803_BIDIRECTMODE_VERT);
}


/** \brief Shows possible cursormodes
 *
 */
void Demo_CursorMode(LCD1803_t *disp) {
    LCD1803_Clear(disp);

    PutString(disp, "Set_CursorMode:");

    for(int repeat=0; repeat < 5; repeat++) {
        LCD1803_MoveXY(disp, 0, 2);
        LCD1803_Set_CursorMode(disp, LCD1803_CURSORMODE_OFF);
        PutString(disp, "OFF..");
        HAL_Delay(2000);
        LCD1803_Set_CursorMode(disp, LCD1803_CURSORMODE_ON);
        LCD1803_MoveXY(disp, 0, 2);
        PutString(disp, "ON...");
        HAL_Delay(2000);
        LCD1803_Set_CursorMode(disp, LCD1803_CURSORMODE_BLINK);
        LCD1803_MoveXY(disp, 0, 2);
        PutString(disp, "BLINK");
        HAL_Delay(2000);
        // INV not working yet
    }

    LCD1803_Set_CursorMode(disp, LCD1803_CURSORMODE_OFF);
}


/** \brief Shows possible entrymodes
 *
 */
void Demo_EntryMode(LCD1803_t *disp) {
    static const char Str_Dec[] = "Decrement";
    static const char Str_Inc[] = "Increment";

    LCD1803_Set_CursorMode(disp, LCD1803_CURSORMODE_BLINK);

    for(int repeat=0; repeat < 5; repeat++) {
        LCD1803_Clear(disp);
        PutString(disp, "Set_EntryMode:")

        LCD1803_MoveXY(disp, 10, 2);
        LCD1803_Set_EntryMode(disp, LCD1803_ENTRYMODE_DEC);

        for(int c=0; Str_Dec[c] != '\0'; c++) {
            LCD1803_Write(disp, (uint8_t *) &Str_Dec[c], 1);
            HAL_Delay(300);
        }

        LCD1803_MoveXY(disp, 10, 3);
        LCD1803_Set_EntryMode(disp, LCD1803_ENTRYMODE_INC);

        for(int c=0; Str_Inc[c] != '\0'; c++) {
            LCD1803_Write(disp, (uint8_t *) &Str_Inc[c], 1);
            HAL_Delay(300);
        }
    }
}


/** \brief Shows possible cursor shift
 *
 */
void Demo_ShiftCursor(LCD1803_t *disp) {
    LCD1803_Set_CursorMode(disp, LCD1803_CURSORMODE_ON);
    LCD1803_Clear(disp);
    PutString(disp, "ShiftCursor:");

    ClearLine(disp, 2);
    LCD1803_MoveXY(disp, 0, 2);
    PutString(disp, "Shift Left");
    for(int repeat=0; repeat < 100; repeat++) {
        LCD1803_ShiftCursor(disp, LCD1803_SHIFT_LEFT);
        HAL_Delay(100);
    }

    ClearLine(disp, 2);
    LCD1803_MoveXY(disp, 0, 2);
    PutString(disp, "Shift Right");
    for(int repeat=0; repeat < 100; repeat++) {
        LCD1803_ShiftCursor(disp, LCD1803_SHIFT_RIGHT);
        HAL_Delay(100);
    }

    LCD1803_Set_CursorMode(disp, LCD1803_CURSORMODE_OFF);
}


/** \brief Shows possible display shift
 *
 */
void Demo_ShiftDisplay(LCD1803_t *disp) {
    LCD1803_Clear(disp);
    PutString(disp, "Shift Display:");
    LCD1803_MoveXY(disp, 0, 2);
    PutString(disp, "Note: Addressing");
    LCD1803_MoveXY(disp, 0, 3);
    PutString(disp, "does not change!");

    LCD1803_MoveXY(disp, 0, 1);
    PutString(disp, "Shift LEFT");

    for(int repeat=0; repeat < 80; repeat++) {
        LCD1803_ShiftDisplay(disp, LCD1803_SHIFT_LEFT);
        HAL_Delay(200);
    }

    LCD1803_MoveXY(disp, 0, 1);
    PutString(disp, "Shift RIGHT");

    for(int repeat=0; repeat < 80; repeat++) {
        LCD1803_ShiftDisplay(disp, LCD1803_SHIFT_RIGHT);
        HAL_Delay(200);
    }
}


/** \brief Shows single line auto shift
 *
 */
void Demo_AutoShift(LCD1803_t *disp) {
    static const char Str[] = "Demo: Single line shift... ";
    int pos;

    LCD1803_Set_CursorMode(disp, LCD1803_CURSORMODE_OFF);

    LCD1803_Clear(disp);
    PutString(disp, "Set_ShiftMode:");
    LCD1803_MoveXY(disp, 0, 1);
    PutString(disp, "Set on/off per line");
    LCD1803_MoveXY(disp, 0, 2);
    PutString(disp, "and auto possible");

    LCD1803_MoveXY(disp, 0, 3);
    LCD1803_Set_ShiftMode(disp, LCD1803_SHIFTMODE_NORM, 0x08);
    LCD1803_Set_ShiftOn(disp, 1);

    // Note: Although display shifts, cursor MOVES also to next position.
    // There is no way to reset the shift, so we have to keep track of the
    // Position to shift it back to normal position after

    pos = 0;
    for(int repeat=0; repeat < 5; repeat++) {
        for(int c = 0; Str[c] != '\0'; c++) {
            LCD1803_Write(disp, (uint8_t *) &Str[c], 1);
            HAL_Delay(200);
            pos++;
            if(pos == 20) {
                pos = 0;
                LCD1803_MoveXY(disp, 0, 3);
            }
        }
    }

    while(pos++ < 20) {                       // Fixup to have original position after all...
        LCD1803_Write(disp, (uint8_t *) " ", 1);
    }
    LCD1803_Set_ShiftOn(disp, 0);
}


/** \brief Shows smooth scroll
 *
 */
void Demo_SmoothScroll(LCD1803_t *disp) {
    static const char Str[] = "Demo: Smooth scroll... ";
    int pos;

    LCD1803_Set_CursorMode(disp, LCD1803_CURSORMODE_OFF);
    LCD1803_Clear(disp);
    PutString(disp, "ScrollDisplay:");
    LCD1803_MoveXY(disp, 0, 1);
    PutString(disp, "Use dot scroll for");
    LCD1803_MoveXY(disp, 0, 2);
    PutString(disp, "Smooth scrolling");

    LCD1803_Set_ShiftMode(disp, LCD1803_SHIFTMODE_DOT, 0x08);
    LCD1803_Set_ShiftMode(disp, LCD1803_SHIFTMODE_NORM, 0x08);
    LCD1803_Set_ShiftOn(disp, 1);

    pos = 0;
    LCD1803_MoveXY(disp, 0, 3);
    for(int repeat=0; repeat < 5; repeat++) {
        for(int c = 0; Str[c] != '\0'; c++) {
            LCD1803_Write(disp, (uint8_t *) &Str[c], 1);
            for(uint8_t S = 0; S < 5; S++) {
                LCD1803_ScrollDisplay(disp, S);
                HAL_Delay(100);
            }
            pos++;
            if(pos == 20) {
                pos = 0;
                LCD1803_MoveXY(disp, 0, 3);
            }
        }
    }

    LCD1803_ScrollDisplay(disp, 0);
    while(pos++ < 20) {                       // Fixup to have original position after all...
        LCD1803_Write(disp, (uint8_t *) " ", 1);
    }
    LCD1803_Set_ShiftOn(disp, 0);
}


/** \brief Shows double height modes
 *
 */
void Demo_DH(LCD1803_t *disp) {
    LCD1803_Clear(disp);
    PutString(disp, "Set_DHMode:");
    LCD1803_MoveXY(disp, 0, 1);
    PutString(disp, "Line2");
    LCD1803_MoveXY(disp, 0, 2);
    PutString(disp, "Line3");
    LCD1803_MoveXY(disp, 0, 3);
    PutString(disp, "Line4");

    for(int repeat=0; repeat < 5; repeat++) {
        for(uint8_t mode=0; mode <= 4; mode++) {
            LCD1803_Set_DHMode(disp, mode);
            HAL_Delay(2000);
        }
    }
    LCD1803_Set_DHMode(disp, LCD1803_DH_NONE);
}


/** \brief Shows different ROM char sets
 *
 */
void Demo_Rom(LCD1803_t *disp) {
    uint8_t buff[4], c;
    int num;

    LCD1803_Set_CursorMode(disp, LCD1803_CURSORMODE_ON);

    LCD1803_Clear(disp);
    PutString(disp, "Set_Rom:");
    for(int i = 0; i <= 240; i+=48) {
        for(int j = 0 ; j < 3; j++) {
            LCD1803_MoveXY(disp, 0, j+1);
            num = (i+j*16)&0xff;
            buff[0] = '0'+num/100;
            buff[1] = '0'+(num%100)/10;
            buff[2] = '0'+num%10;
            buff[3] = ':';
            LCD1803_Write(disp, buff, 4);
            for(int k = num; k < num+16; k++) {
                c = k;
                LCD1803_Write(disp, &c, 1);
            }
        }

        for(uint8_t rom = 0; rom < 3; rom++) {
            LCD1803_Set_Rom(disp, rom);
            LCD1803_MoveXY(disp, 9, 0);
            c = rom+'A';
            LCD1803_Write(disp, &c, 1);
            HAL_Delay(3000);
        }
    }
}


/** \brief Shows user definable characters & blinking
 *
 */
void Demo_CG(LCD1803_t *disp) {
    uint8_t c;
    static uint8_t cd[] = {
        0b00010001,
        0b00001110,
        0b00011111,
        0b00010101,
        0b00011111,
        0b00001010,
        0b00011011,
        0,
        0b00011011,
        0b00000000,
        0b00000100,
        0b00000100,
        0b00010001,
        0b00010001,
        0b00001110,
        0,
        0b00011011,
        0b00000000,
        0b00000100,
        0b00000100,
        0b00000000,
        0b00001110,
        0b00010001,
        0,
        0b00010000,
        0b00010100,
        0b00011100,
        0b00000100,
        0b00010101,
        0b00001110,
        0b00000100,
        0,
        0b00001001,
        0b00010010,
        0b00001001,
        0b00010010,
        0b00000000,
        0b00011111,
        0b00001110,
        0,
        0b00001110,
        0b00010001,
        0b00010011,
        0b00010111,
        0b00010011,
        0b00010001,
        0b00001110,
        0,
        0b00001110,
        0b00010001,
        0b00011001,
        0b00011101,
        0b00011001,
        0b00010001,
        0b00001110,
        0,
        0b00001110,
        0b00011011,
        0b00011110,
        0b00011100,
        0b00011110,
        0b00011111,
        0b00001110,
        0
    };
    LCD1803_Set_CursorMode(disp, LCD1803_CURSORMODE_OFF);
    LCD1803_Clear(disp);
    PutString(disp, "Set_CGRAM:");
    LCD1803_MoveXY(disp, 0, 1);
    PutString(disp, "User defined chars:")
    LCD1803_MoveXY(disp, 0, 2);

    for(uint8_t i = 0; i < 8; i++) {
        LCD1803_Write(disp, &i, 1);
        LCD1803_Write(disp, (uint8_t *) " ", 1);
    }

    LCD1803_CGRAMADDR(disp, 0);
    for(int i = 0; i < 64; i++) {
            LCD1803_Write(disp, &cd[i], 1);
            HAL_Delay(200);
    }

    HAL_Delay(5000);

    LCD1803_MoveXY(disp, 0, 3);
    PutString(disp, "They can also blink!");

    for(int i = 0; i < 64; i++) {
        cd[i] ^= 0xc0;
    }

    LCD1803_Set_BlinkOn(disp, 1);

    LCD1803_CGRAMADDR(disp, 0);
    for(int i = 0; i < 64; i++) {
            LCD1803_Write(disp, &cd[i], 1);
            HAL_Delay(200);
    }

    HAL_Delay(5000);

    LCD1803_CGRAMADDR(disp, 0);
    c = 0x00;
    for(int i = 0; i < 64; i++) {
            LCD1803_Write(disp, &c, 1);
            HAL_Delay(200);
    }
}


/** \brief Shows reverse display
 *
 */
void Demo_Reverse(LCD1803_t *disp) {
    LCD1803_Clear(disp);
    LCD1803_MoveXY(disp, 4, 1);
    PutString(disp, "Display can");
    LCD1803_MoveXY(disp, 4, 2);
    PutString(disp, "be inverted");

    for(int repeat = 0; repeat < 10; repeat++) {
        LCD1803_Set_ReverseOn(disp, 1);
        HAL_Delay(500);
        LCD1803_Set_ReverseOn(disp, 0);
        HAL_Delay(500);
    }
}


/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @param  None
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler */
  /* User can add his own implementation to report the HAL error return state */
  while(1)
  {
  }
  /* USER CODE END Error_Handler */
}

#ifdef USE_FULL_ASSERT

/**
   * @brief Reports the name of the source file and the source line number
   * where the assert_param error has occurred.
   * @param file: pointer to the source file name
   * @param line: assert_param error line source number
   * @retval None
   */
void assert_failed(uint8_t* file, uint32_t line)
{
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
    ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */

}

#endif

/**
  * @}
  */

/**
  * @}
*/

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
